# Copyright (c) 2005-2006, Samuel Schluep, ETH Zurich and Infrae
# All rights reserved. See also LICENSE.txt

# Zope
from StringIO import StringIO
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Globals import InitializeClass
from Products.ParsedXML.ExtraDOM import writeStream
from Products.ParsedXML.ParsedXML import ParsedXML
from Products.ParsedXML.ManageableDOM import DOMManageable
# Silva
from Products.Silva.Asset import Asset
from Products.Silva.interfaces import IAsset
from Products.Silva import SilvaPermissions
from Products.Silva.helpers import add_and_edit, translateCdata
from Products.Silva.ImporterRegistry import get_xml_id, get_xml_title
from Products.Silva import mangle
# Silva Document
from Products.SilvaDocument.EditorSupportNested import mixedContentSupportRegistry
# SilvaGlossary
import helpers


icon = "www/glossary_entry.png"

class GlossaryEntry(Asset):
    """A Glossary Entry is an asset and is used to specify a single 
       glossary entry with a glossary term and its description. The
       entries are referred to in documents by links. Using the Silva 
       Gossary product, these links will be rendered specially (e.g.
       showing the glossary entry as a tooltip). Further all glossary
       entries in a Silva Publication or Folder can be summarized on a
       single page using the Silva Glossary content type.
    """
    security = ClassSecurityInfo()

    meta_type = "Silva Glossary Entry"

    def __init__(self, id, title):
        GlossaryEntry.inheritedAttribute('__init__')(self, id, title)
        self.description = ParsedXML('description', '<p></p>')

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_description')
    def get_description(self):
        """Returns the glossary description string

        The description string is returned using "Silva editable markup"
        """
        supp = self.service_editorsupport.getMixedContentSupport(
            self, self.description.documentElement)
        return supp.renderEditable()

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'set_description')
    def set_description(self, description):
        """Sets the glossary description

        description -- utf-8 String, since parse changes it to Unicode.
        """
        supp = self.service_editorsupport.getMixedContentSupport(
            self, self.description.documentElement)
        supp.parse(description.encode('utf-8'))
        #
        self.service_silva_glossary.reset_cache(self)

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'render_description')
    def render_description(self):
        """Returns the glossary description string as HTML.
        """
        supp = self.service_editorsupport.getMixedContentSupport(
            self, self.description.documentElement)
        return supp.renderHTML()

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'to_xml')
    def to_xml(self, context):
        """Render object to XML.
        """
        f = context.f
        f.write(
            '<silva_glossaryentry id="%s"><title>%s</title><description>' % (
                self.id, translateCdata(self.get_title()) ))
        self.description.documentElement.writeStream(f)
        f.write('</description></silva_glossaryentry>')


InitializeClass(GlossaryEntry)


manage_addGlossaryEntryForm = PageTemplateFile(
    "www/glossaryEntryAdd",
    globals(),
    __name__='manage_addGlossaryEntryForm')

def manage_addGlossaryEntry(self, id, title, description='', REQUEST=None):
    """Add a Silva Glossary Entry."""
    if id == '':
        id = title.lower()
        id = helpers.Id(self.get_container(), id)
        id.cook_valid()
    else:
        id = helpers.Id(self.get_container(), id)
    if not id.isValid():
        return
    id = str(id)

    object = GlossaryEntry(id, title)
    self._setObject(id, object)
    object = getattr(self, id)
    object.set_title(title)
    object.set_description(description)
    add_and_edit(self, id, REQUEST)
    return ''


def xml_import_handler(object, node):
    id = get_xml_id(node)
    title = get_xml_title(node)

    id = str(mangle.Id(object, id).unique())
    object.manage_addProduct['SilvaGlossary'].manage_addGlossaryEntry(id, title)

    newdoc = getattr(object, id)
    newdoc.sec_update_last_author_info()

    for child in node.childNodes:
        if child.nodeName == u'doc':
            childxml = writeStream(child).getvalue().encode('utf8')
            object.description.manage_edit(childxml) # expects utf8
        elif hasattr(newdoc, 'set_%s' % child.nodeName.encode('utf8')) \
                and child.nodeValue:
            getattr(newdoc, 'set_%s' % child.nodeName.encode('utf8'))(
                child.nodeValue.encode('utf8'))

    return newdoc
